<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Controller\Split;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Montonio\Hirepurchase\Exception\PaymentCancelledStatusException;
use Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\ResponseHandlerInterface;
use Montonio\Hirepurchase\Gateway\Split\ResponseHandler\TokenParser;
use Montonio\Hirepurchase\Model\Action\RetrieveOrderByIncrementId;
use Psr\Log\LoggerInterface;

/**
 * Class responsible for handling API hook POST action.
 * todo: Magento\Framework\App\Action\Action usage should be eliminated on 2.4+ Magento versions.
 */
class Hook extends Action implements CsrfAwareActionInterface, HttpPostActionInterface
{
    /**
     * @var ResponseHandlerInterface
     */
    private $responseHandler;

    /**
     * @var TokenParser
     */
    private $tokenParser;

    /**
     * @var RetrieveOrderByIncrementId
     */
    private $retrieveOrderByIncrementId;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param Context $context
     * @param ResponseHandlerInterface $responseHandler
     * @param TokenParser $tokenParser
     * @param RetrieveOrderByIncrementId $retrieveOrderByIncrementId
     * @param LoggerInterface $logger
     */
    public function __construct(
        Context $context,
        ResponseHandlerInterface $responseHandler,
        TokenParser $tokenParser,
        RetrieveOrderByIncrementId $retrieveOrderByIncrementId,
        LoggerInterface $logger
    ) {
        parent::__construct($context);

        $this->responseHandler = $responseHandler;
        $this->tokenParser = $tokenParser;
        $this->retrieveOrderByIncrementId = $retrieveOrderByIncrementId;
        $this->logger = $logger;
    }

    /**
     * Handle API hook POST request.
     *
     * @return ResultInterface
     */
    public function execute(): ResultInterface
    {
        $resultJson = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        $result =   [
                        'success' => true,
                        'error' => false,
                        'data' => [],
                        'message' => 'finalized'
                    ];
        try {
            $tokenData = $this->tokenParser->parseToken($this->_request->getParam('payment_token', ''));

            $orderId = false;
            //sandbox adds _suffix to make testing possible
            if (!empty($tokenData['merchant_reference'])) {
                $orderId = explode('_', $tokenData['merchant_reference']);
                $orderId = $orderId[0];
            }
            $order = $this->retrieveOrderByIncrementId->execute($orderId);
            $this->responseHandler->handle($order, $tokenData);
        } catch (PaymentCancelledStatusException $exception) {
            $result =   [
                            'success' => true,
                            'error' => false,
                            'data' => [],
                            'message' => 'abandoned'
                        ];
        } catch (\Throwable $exception) {
            $this->logger->info('Error handling API return action.', ['exception' => $exception->getMessage()]);
            $result =   [
                            'success' => false,
                            'error' => true,
                            'data' => [],
                            'message' => $exception->getMessage()
                        ];
        }

        return $resultJson->setData($result);
    }

    /**
     * @inheritdoc
     */
    public function createCsrfValidationException(RequestInterface $request): ? InvalidRequestException
    {
        return null;
    }

    /**
     * @inheritdoc
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
}
