<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Controller\Split;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Montonio\Hirepurchase\Exception\PaymentCancelledStatusException;
use Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\ResponseHandlerInterface;
use Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\RestoreLastCartSession;
use Montonio\Hirepurchase\Gateway\Payments\ResponseHandler\RestoreSuccessSession;
use Montonio\Hirepurchase\Gateway\Split\ResponseHandler\TokenParser;
use Montonio\Hirepurchase\Model\Action\RetrieveOrderByIncrementId;

use Psr\Log\LoggerInterface;

/**
 * Class responsible for handling API return action.
 * todo: Magento\Framework\App\Action\Action usage should be eliminated on 2.4+ Magento versions.
 */
class Index extends Action implements HttpGetActionInterface
{
    private const SUCCESS_REDIRECT_PATH = 'checkout/onepage/success';
    private const FAILURE_REDIRECT_PATH = 'checkout/cart';

    /**
     * @var ResponseHandlerInterface
     */
    private $responseHandler;

    /**
     * @var TokenParser
     */
    private $tokenParser;

    /**
     * @var RetrieveOrderByIncrementId
     */
    private $retrieveOrderByIncrementId;

    /**
     * @var RestoreLastCartSession
     */
    private $restoreLastCartSession;

    /**
     * @var RestoreSuccessSession
     */
    private $restoreSuccessSession;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     *
     * @param Context $context
     * @param ResponseHandlerInterface $responseHandler
     * @param TokenParser $tokenParser
     * @param RetrieveOrderByIncrementId $retrieveOrderByIncrementId
     * @param RestoreLastCartSession $restoreLastCartSession
     * @param RestoreSuccessSession $restoreSuccessSession
     * @param LoggerInterface $logger
     */
    public function __construct(
        Context $context,
        ResponseHandlerInterface $responseHandler,
        TokenParser $tokenParser,
        RetrieveOrderByIncrementId $retrieveOrderByIncrementId,
        RestoreLastCartSession $restoreLastCartSession,
        RestoreSuccessSession $restoreSuccessSession,
        LoggerInterface $logger
    ) {
        parent::__construct($context);
        $this->messageManager = $context->getMessageManager();
        $this->responseHandler = $responseHandler;
        $this->tokenParser = $tokenParser;
        $this->retrieveOrderByIncrementId = $retrieveOrderByIncrementId;
        $this->restoreLastCartSession = $restoreLastCartSession;
        $this->restoreSuccessSession = $restoreSuccessSession;
        $this->logger = $logger;
    }

    /**
     * Handle API return action.
     *
     * @return void
     */
    public function execute(): void
    {
        $redirectPath = self::SUCCESS_REDIRECT_PATH;
        try {
            $tokenData = $this->tokenParser->parseToken($this->_request->getParam('payment_token', ''));

            $orderId = false;
            //sandbox adds _suffix to make testing possible
            if (!empty($tokenData['merchant_reference'])) {
                $orderId = explode('_', $tokenData['merchant_reference']);
                $orderId = $orderId[0];
            }

            $order = $this->retrieveOrderByIncrementId->execute($orderId);
            $this->responseHandler->handle($order, $tokenData);
            $this->restoreSuccessSession->execute($order);
        } catch (PaymentCancelledStatusException $exception) {
            $this->restoreLastCartSession->execute();
            $this->messageManager->addSuccessMessage($exception->getMessage());
            $redirectPath = self::FAILURE_REDIRECT_PATH;
        } catch (\Throwable $exception) {
            $this->logger->error('Error handling API return action.', ['exception' => $exception->getMessage()]);
            $redirectPath = self::FAILURE_REDIRECT_PATH;
        }
        $this->_redirect($redirectPath);
    }
}
