<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Gateway\Hirepurchase\Request;

use Magento\Payment\Gateway\ConfigInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;

/**
 * @inheritdoc
 */
class RedirectRequest implements BuilderInterface
{
    /**
     * @var ConfigInterface
     */
    private $config;

    /**
     *
     * @param \Magento\Payment\Gateway\ConfigInterface $config
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface
     * @param \Magento\Framework\Pricing\PriceCurrencyInterface $priceCurrencyInterface
     * @param \Magento\Framework\UrlInterface $urlInterface
     */
    public function __construct(
        \Magento\Payment\Gateway\ConfigInterface $config,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface,
        \Magento\Framework\Pricing\PriceCurrencyInterface $priceCurrencyInterface,
        \Magento\Framework\UrlInterface $urlInterface
    ) {
        $this->config = $config;
        $this->timezoneInterface = $timezoneInterface;
        $this->priceCurrencyInterface = $priceCurrencyInterface;
        $this->urlInterface = $urlInterface;
    }

    /**
     * Builds the request
     *
     * @param array $buildSubject
     * @return array
     */
    public function build(array $buildSubject)
    {

        if (!isset($buildSubject['payment'])
            || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new \InvalidArgumentException('Payment data object should be provided');
        }

        /** @var PaymentDataObjectInterface $payment */
        $payment = $buildSubject['payment'];
        $order = $payment->getOrder();
        $address = $order->getShippingAddress();

        $storeId = $order->getStoreId();
        $environment = $this->config->getValue('environment', $storeId);

        $streetLines = trim($address->getStreetLine1()) . (
            ($address->getStreetLine2() !='') ? trim($address->getStreetLine2()) : '');
        $addressFields = [
            'customer_first_name' => trim($address->getFirstname()),
            'customer_last_name' => trim($address->getLastname()),
            'customer_email' => trim($address->getEmail()),
            'customer_phone' => trim($address->getTelephone()),
            'customer_city' => trim($address->getCity()),
            'customer_address' => $streetLines,
            'customer_postal_code' => trim($address->getPostcode())
        ];

        //to provide return url we sign it with out private key
        //so we can check later if stuff sent back to us is valid
        $returnUrl = $this->config->getValue('return_url', $storeId);
        $hookUrl = $this->config->getValue('return_hook_url', $storeId);
        $orderIncrementId = $order->getOrderIncrementId();
        $privKey = $this->config->getValue($environment.'_private_key', $storeId);
        $routeParams = [
            'ref' => $orderIncrementId,
            'sig' => $this->config->generateSignature([$orderIncrementId], $privKey)
        ];
        $callbackUrl = $this->urlInterface->getUrl($returnUrl."/", $routeParams);
        $callbackHookUrl = $this->urlInterface->getUrl($hookUrl."/", $routeParams);

        $requestParams = [
            'origin' =>'online',
            'merchant_reference' => $orderIncrementId,
            'callback_url' => $callbackUrl,
            'notification_url' => $callbackHookUrl,
            'timestamp' => $this->timezoneInterface->date()->format('c')
        ];

        foreach ($order->getItems() as $item) {

            //we need only single simple, configurable and bundle parent items
            if ($item->getProductId() !== $item->getData('product_options')['info_buyRequest']['product']
            ) {
                continue;
            }

            //for configurable name we get the option name
            $name = (
                $item->getProductType() !== 'configurable') ?
                (string)$item->getName() :
                (string)$item->getData('product_options')['simple_name'];

            $requestParams['products'][] = [
                'product_name' => $name,
                'quantity' => (int)$item->getQtyOrdered(),
                'product_price' => sprintf("%.2F", 0)
            ];

        }

        //we report the toal price only since montonio prefers no negative total or item prices
        $requestParams['products'][] = [
            'product_name' => 'Cart total',
            'quantity' => 1,
            'product_price' => sprintf("%.2F", $order->getGrandTotalAmount())
        ];

        $requestParams = array_merge($requestParams, $addressFields);

        $headers = [
            'Content-Type' => 'application/json',
            'X-Access-Key' => $this->config->getValue($environment.'_public_key', $storeId),
            'X-Signature' => $this->config->generateSignature($requestParams, $privKey),
        ];

        return [
            'url' => $this->config->getValue($environment.'_api_url', $storeId).'/applications',
            'headers' => $headers,
            'params' => $requestParams
        ];
    }
}
