<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Gateway\Split\Command;

/**
 * @inheritdoc
 */
class DataCommand implements \Magento\Payment\Gateway\CommandInterface
{
    /**
     *
     * @param \Montonio\Hirepurchase\Gateway\Split\Config\Config $config
     * @param \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Magento\Framework\Locale\Resolver $localeResolver
     */
    public function __construct(
        \Montonio\Hirepurchase\Gateway\Split\Config\Config $config,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezoneInterface,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Framework\Locale\Resolver $localeResolver
    ) {
            $this->config = $config;
            $this->timezoneInterface = $timezoneInterface;
            $this->urlInterface = $urlInterface;
            $this->localeResolver = $localeResolver;
    }

    /**
     * return payment request data
     *
     * @param array $commandSubject
     * @return string|Command\ResultInterface
     */
    public function execute(array $commandSubject)
    {
        if (!isset($commandSubject['order']) || !isset($commandSubject['method'])
        ) {
            throw new \InvalidArgumentException(__('Verify  payment request data $order, $method'));
        }
        $redirectUrl = $this->buildRequestData($commandSubject['order'], $commandSubject['method']);
        return $redirectUrl;
    }

    protected function buildRequestData($order, $method)
    {

        $storeId = $order->getStoreId();
        $environment = $this->config->getValue('environment', $storeId);

        $data = [];
        /* system data */
        $data['loan_type'] = 'slice';
        $data['iat'] = $this->timezoneInterface->date()->format('U');
        $data['exp'] = $this->timezoneInterface->date()->format('U') + (60 * 10);
        $data['access_key'] = $this->config->getValue($environment.'_public_key', $storeId);

        $returnUrl = $this->config->getValue('return_url', $storeId);
        $hookUrl = $this->config->getValue('return_hook_url', $storeId);

        $data['merchant_return_url'] = $this->urlInterface->getUrl($returnUrl);
        $data['merchant_notification_url'] = $this->urlInterface->getUrl($hookUrl);

        //loan period if any
        //codes are defined in aspspsCommand and combined = null otherwise last int after dash is the period
        $codeParts = explode('_', $order->getPayment()
                                ->getMethodInstance()->getCode());
        if (is_array($codeParts) && end($codeParts) !== 'combined') {
            $data['preselected_loan_period'] =  end($codeParts) ;
        }

        $storeLocale = $this->localeResolver->getLocale();

        //montonio only supports following locales
        $locales = [
            'en_US' => 'en_US',
            'et_EE' => 'et',
            'lt_LT' => 'lt',
            'ru_RU' => 'ru'
        ];

        $localeCode = (isset($locales[$storeLocale])) ? $locales[$storeLocale] : $storeLocale;

        $data['preselected_locale'] = $localeCode;

        /*order data*/
        $data['currency'] = $order->getOrderCurrency()->toString();

        if ($environment == 'sandbox') {
            $data['merchant_reference'] = $order->getIncrementId().'_'.rand(1000, 4);
        } else {
            $data['merchant_reference'] = $order->getIncrementId();
        }

        $data['payment_information_unstructured'] = (string)__('Payment for order %1', $order->getIncrementId());

        if ($order->getCustomerEmail()) {
            $data['checkout_email'] = $order->getCustomerEmail();
        }

        if ($order->getBillingAddress()->getFirstname()) {
            $data['checkout_first_name'] = $order->getBillingAddress()->getFirstname();
        }

        if ($order->getBillingAddress()->getLastname()) {
            $data['checkout_last_name'] = $order->getBillingAddress()->getLastname();
        }

        if ($order->getBillingAddress()->getTelephone()) {
            $data['checkout_phone_number'] =  $order->getBillingAddress()->getTelephone();
        }

        if ($order->getBillingAddress()->getCity()) {
            $data['checkout_city'] = $order->getBillingAddress()->getCity();
        }

        if (is_array($order->getBillingAddress()->getStreet()) && $order->getBillingAddress()->getStreet()[0]) {
            $data['checkout_address'] = $order->getBillingAddress()->getStreet()[0];
        } elseif (!empty($order->getBillingAddress()->getStreet())) {
            $data['checkout_address'] = $order->getBillingAddress()->getStreet();
        }

        if ($order->getBillingAddress()->getPostcode()) {
            $data['checkout_postal_code'] = $order->getBillingAddress()->getPostcode();
        }

        foreach ($order->getItems() as $item) {

            //we need only single simple, configurable and bundle parent items
            if ($item->getProductId() !== $item->getData('product_options')['info_buyRequest']['product']) {
                continue;
            }

            //for configurable name we get the option name
            $name = ($item->getProductType() !== 'configurable') ?
                (string)$item->getName() :
                (string)$item->getData('product_options')['simple_name'];

                $data['checkout_products'][] = [
                'product_name' => $name,
                'quantity' => (int)$item->getQtyOrdered(),
                'product_price' => sprintf("%.2F", 0)
                ];

        }

        //we report the toal price only since montonio prefers no negative total or item prices
        $data['checkout_products'][] = [
            'product_name' => 'Cart total',
            'quantity' => 1,
            'product_price' => sprintf("%.2F", $order->getGrandTotal())
        ];

        $privateKey = $this->config->getValue($environment.'_private_key', $storeId);
        $applicationUrl= $this->config->getValue($environment.'_application_url', $storeId) . '/?payment_token=';
        $jwtToken = $this->config->generateSignature($data, $privateKey);

        $data = $applicationUrl . $jwtToken;

        return $data;
    }
}
